module phaseshift_mod

  use sep
  use fourier

  implicit none
  integer, private                              :: nz, nt, nx
  real, private                                 :: dz, dt, dx, v, qi
  complex, dimension(:,:), allocatable, private :: cmodl, cdata, ctemp
  real, allocatable, dimension(:), private	    :: depth
  logical, allocatable, dimension(:), private	:: depth_ok

  contains

  subroutine phase1_init(nz_in, nt_in, nx_in, dz_in, dt_in, dx_in, v_in, depth_in)
    integer                            :: nz_in, nt_in, nx_in
    real                               :: dz_in, dt_in, dx_in, v_in
    real, allocatable, dimension(:)    :: depth_in
    nz = nz_in
    nt = nt_in
    nx = nx_in
    dz = dz_in
    dt = dt_in
    dx = dx_in
    v = v_in
    allocate(cmodl(nt,nx), cdata(nt,nx), ctemp(nt,nx))
    allocate(depth(nx),depth_ok(nx))
    depth = 0.
    cmodl = 0.
    cdata = 0.
    ctemp = 0.
    depth = depth_in
    depth_ok = .false.
    qi = 0.5/(nt*dt)
    call ft_init(nt, nx)
  end subroutine

!===============================================================


	subroutine phase1_call(adj,add,mm,dd)
    		logical,intent(in)   :: adj,add
   			real,dimension(:) 	 :: mm,dd
    		call phase1(adj,add,mm,dd)
  	end subroutine phase1_call

  subroutine phase1(adj, add, modl, data)  
    logical, intent(in)   :: adj, add
    real, dimension(nt,nx):: modl, data
    integer               :: iz, ix, it, nw, iw, ikx
    real                  :: pi, signt, signx, rv2
    real                  :: w0, dw, w, kx0, dkx, kx, kz2
    complex               :: phase

    pi = 4.*atan(1.)
    signt =1.
    signx =-1.
    rv2=1./(v*v)

    nw = nt
    w0 = -pi/dt
    dw = 2.*pi/(nt*dt)
    kx0 = -pi/dx
    dkx= 2.*pi/(nx*dx)


    if(.not. add) then
      if(adj) then 
        modl = 0.
      else 		    
        data = 0.
      endif
    endif

    if(adj) then ! DOWNWARD CONTINUATION (migration)
      depth_ok = .false.
      cdata = data
      call ft1axis(.false., signt, cdata) ! FFT over t
      call ft2axis(.false., signx, cdata) ! FFT over x
      do iz=1,nz
      	if(mod(iz,10).eq.0) write(0,*)"downward continuating depth:",iz*dz
      	!$OMP PARALLEL DO private(ikx,iw,kz2,w,kx,phase)
        do ikx=1,nx
          if(depth_ok(ikx)) cycle
          kx = kx0 + (ikx-1)*dkx
          do iw=1,nw
            w = w0 + (iw-1)*dw
            kz2 = w*w*rv2 - kx*kx
            if(kz2 >0) then
              phase = cexp(cmplx(0., signum(w)*dz*sqrt(kz2)))
              !phase = eiktau(dz*v,w,v*kx,qi)
              cdata(iw,ikx) = cdata(iw,ikx)*phase
            endif
          enddo
          if(nint(depth(ikx)/dz).eq.iz) then
          	depth_ok(ikx)=.true.
          	ctemp(:,ikx) = cdata(:,ikx)
          endif
        enddo
        !$OMP END PARALLEL DO
      enddo
      call ft1axis(.true., signt, ctemp) ! inverse FFT in t
      call ft2axis(.true., signx, ctemp) ! inverse FFT in x
      modl = modl + real(ctemp)
      
      
      
    else ! UPWARD CONTINUATION (modeling)
      depth_ok = .false.
      cmodl = modl
      call ft1axis(.false., signt, cmodl) ! FFT over t
      call ft2axis(.false., signx, cmodl) ! FFT over x
      do iz=1,nz
      	if(mod(iz,10).eq.0) write(0,*)"upward continuating from depth:",iz*dz
      	!$OMP PARALLEL DO private(ikx,iw,kz2,w,kx,phase)
        do ikx=1,nx
          if(depth_ok(ikx)) cycle
          kx = kx0 + (ikx-1)*dkx
          do iw=1,nw
            w = w0 + (iw-1)*dw
            kz2 = w*w*rv2 - kx*kx
            if(kz2 >0) then
              phase = cexp(cmplx(0., -signum(w)*dz*sqrt(kz2)))
              !phase = conjg(eiktau(dz*v,w,v*kx,qi))
              cmodl(iw,ikx) = cmodl(iw,ikx)*phase
            endif
          enddo
          if(nint(depth(ikx)/dz).eq.iz) then
          	depth_ok(ikx)=.true.
          	ctemp(:,ikx) = cmodl(:,ikx)
          endif
        enddo
        !$OMP END PARALLEL DO
      enddo
      call ft1axis(.true., signt, ctemp) ! inverse FFT in t
      call ft2axis(.true., signx, ctemp) ! inverse FFT in x
      data = data + real(ctemp)
    endif
  end subroutine

!===============================================================

  real function signum(x)
    real         :: x
    if(x .eq. 0.) then
      signum = 0. 
    else 
      signum = x/abs(x)
    endif
  end function
  
  complex function eiktau(dt,w,vkx,qi)
  	real		:: dt, w, vkx, qi
  	eiktau = cexp(-dt*csqrt(cmplx(qi,-w)**2+ vkx*vkx))
  	return
  end function

end module
