%File for creating the synthetic model necessary for the up-down separation with downward
%continuation
t0=1.5;%one-way traveltime to sea bottom
t1=0.5;%two-wave traveltime from sea bottom to first reflector
dt=0.004;%s time sampling
v1=1500;%m/s sea water wave velocity
v2=2000;%m/s first layer
doff=100;
offset=(-10000:doff:10000);


%Compute traveltime
t1x=sqrt(t0^2+offset.^2/v1^2);%direct arrival
t2x=sqrt((t1)^2+offset.^2/(4*v2^2));%first reflection to the surface
t1m=sqrt(9*t0^2+offset.^2/v1^2);%first multiple


%create data without statics
tmax=max([t1x t2x t1m])+0.5;
impulse=zeros(floor(tmax/dt+1),length(offset));
amp=1;
for j=1:size(impulse,2)
    nsamp=floor(t2x(j)/dt);
    w=t2x(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
end
rick=ricker(10,101,0.004,51*dt);
data=zeros(floor(tmax/dt+1),length(offset));
for j=1:size(impulse,2)
    data(:,j)=conv(impulse(:,j),rick,'same');
end
%writing the .H@ data binary
fileID=fopen('/scr1/ettore/data_clean.H@','w');
fwrite(fileID,data,'float32','s');
fclose(fileID);

%writing the .H data file
fileID=fopen('data_clean.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/data_clean.H@');
fprintf(fileID,'n1=%d\n',size(data,1));
fprintf(fileID,'o1=%d\n',0.);
fprintf(fileID,'d1=%d\n',dt);
fprintf(fileID,'n2=%d\n',size(data,2));
fprintf(fileID,'o2=%d\n',min(offset));
fprintf(fileID,'d2=%d\n',doff);
fprintf(fileID,'%s\n','label1="time [s]"');
fprintf(fileID,'%s\n','label2="offset [m]"');
fclose(fileID);





dtn=0.030*(2*rand(length(offset),1)-1);
t2x=t2x+dtn';%imprinting the statics on first reflection
tmax=max([t1x t2x t1m])+0.5;
impulse=zeros(floor(tmax/dt+1),length(offset));
amp=1;
for j=1:size(impulse,2)
    nsamp=floor(t2x(j)/dt);
    w=t2x(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
end
rick=ricker(10,101,0.004,51*dt);
data=zeros(floor(tmax/dt+1),length(offset));
for j=1:size(impulse,2)
    data(:,j)=conv(impulse(:,j),rick,'same');
end
%writing the .H@ data binary
fileID=fopen('/scr1/ettore/data.H@','w');
fwrite(fileID,data,'float32','s');
fclose(fileID);

%writing the .H data file
fileID=fopen('data.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/data.H@');
fprintf(fileID,'n1=%d\n',size(data,1));
fprintf(fileID,'o1=%d\n',0.);
fprintf(fileID,'d1=%d\n',dt);
fprintf(fileID,'n2=%d\n',size(data,2));
fprintf(fileID,'o2=%d\n',min(offset));
fprintf(fileID,'d2=%d\n',doff);
fprintf(fileID,'%s\n','label1="time [s]"');
fprintf(fileID,'%s\n','label2="offset [m]"');
fclose(fileID);

%create constant depth file
%writing the .H@ depth file
depth=zeros(size(data,2)+1000,1);
depth(:)=t0*v1;
fileID=fopen('/scr1/ettore/constdepth.H@','w');
fwrite(fileID,depth,'float32','s');
fclose(fileID);

%writing the .H depth file
fileID=fopen('constdepth.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/constdepth.H@');
fprintf(fileID,'n1=%d\n',size(data,2));
fprintf(fileID,'o1=%d\n',min(offset));
fprintf(fileID,'d1=%d\n',doff);
fprintf(fileID,'%s\n','label1="source position [km]"');
fprintf(fileID,'%s\n','label2="depth [m]"');
fclose(fileID);


%create direct arrival and its mirror
impulse=zeros(floor(tmax/dt+1),length(offset));
amp=1;
for j=1:size(impulse,2)
    nsamp=floor(t1x(j)/dt);
    w=t1x(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
    nsamp=floor(t1m(j)/dt);
    w=t1m(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)-amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)-amp*(1-w);
end
rick=ricker(10,101,0.004,51*dt);
data=zeros(floor(tmax/dt+1),length(offset));
for j=1:size(impulse,2)
    data(:,j)=conv(impulse(:,j),rick,'same');
end

%writing the .H@ down data binary
fileID=fopen('/scr1/ettore/down.H@','w');
fwrite(fileID,data,'float32','s');
fclose(fileID);

%writing the .H data file
fileID=fopen('down.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/down.H@');
fprintf(fileID,'n1=%d\n',size(data,1));
fprintf(fileID,'o1=%d\n',0.);
fprintf(fileID,'d1=%d\n',dt);
fprintf(fileID,'n2=%d\n',size(data,2));
fprintf(fileID,'o2=%d\n',min(offset));
fprintf(fileID,'d2=%d\n',doff);
fprintf(fileID,'%s\n','label1="time [s]"');
fprintf(fileID,'%s\n','label2="offset [m]"');
fclose(fileID);

quit

