%file to try the event separation in the curvelet domain

t0=1.5;%one-way traveltime to sea bottom
t1=0.5;%two-wave traveltime from sea bottom to first reflector
dt=0.004;%s time sampling
v1=2000;%m/s sea water wave velocity
v2=2000;%m/s first layer
doff=50;
offset=(-7000:doff:7000);

t2x=sqrt((t1)^2+offset.^2/(4*v2^2));%first reflection to the surface
t1m=sqrt(t0^2+offset.^2/v1^2);%first multiple
dtn=0.030*(2*rand(length(offset),1)-1);
t2x=t2x+dtn';%imprinting the statics on first reflection
%create data
tmax=max([t2x t1m])+0.05;
impulse=zeros(floor(tmax/dt+1),length(offset));
amp=1;
for j=1:size(impulse,2)
    nsamp=floor(t2x(j)/dt);
    w=t2x(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
    nsamp=floor(t1m(j)/dt);
    w=t1m(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
end
rick=ricker(10,101,0.004,51*dt);
data=zeros(floor(tmax/dt+1),length(offset));
for j=1:size(impulse,2)
    data(:,j)=conv(impulse(:,j),rick,'same');
end

%write gather
%writing the .H@ 
fileID=fopen('/scr1/ettore/events.H@','w');
fwrite(fileID,data,'float32','s');
fclose(fileID);

%writing the .H 
fileID=fopen('events.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/events.H@');
fprintf(fileID,'n1=%d\n',size(data,1));
fprintf(fileID,'o1=%d\n',0.);
fprintf(fileID,'d1=%d\n',dt);
fprintf(fileID,'n2=%d\n',size(data,2));
fprintf(fileID,'o2=%d\n',min(offset));
fprintf(fileID,'d2=%d\n',doff);
fprintf(fileID,'%s\n','label1="time [s]"');
fprintf(fileID,'%s\n','label2="offset [m]"');
fclose(fileID);


addpath('CurveLab-2.1.3/fdct_usfft_matlab')
folder=pwd;
cd CurveLab-2.1.3/fdct_usfft_matlab/
run 'fdct_usfft_path'
cd(folder)
buff_rows=1024;
buff=zeros(buff_rows);
buff(1:size(data,1),1:size(data,2))=data(:,:);
C_data = fdct_usfft(buff,0);
img_data = fdct_usfft_dispcoef(C_data);

%writing the .H@ of img_clean
fileID=fopen('/scr1/ettore/cur_events.H@','w');
fwrite(fileID,img_data,'float32','s');
fclose(fileID);

%writing the .H of img_clean
fileID=fopen('cur_events.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/cur_events.H@');
fprintf(fileID,'n1=%d\n',size(img_data,1));
fprintf(fileID,'o1=%d\n',1.);
fprintf(fileID,'d1=%d\n',1.);
fprintf(fileID,'n2=%d\n',size(img_data,2));
fprintf(fileID,'o2=%d\n',1.);
fprintf(fileID,'d2=%d\n',1.);
fclose(fileID);


%event affected by statics
impulse=zeros(floor(tmax/dt+1),length(offset));
amp=1;
for j=1:size(impulse,2)
    nsamp=floor(t2x(j)/dt);
    w=t2x(j)/dt-nsamp;
    nsamp=nsamp+1;
    impulse(nsamp,j)=impulse(nsamp,j)+amp*w;
    impulse(nsamp+1,j)=impulse(nsamp+1,j)+amp*(1-w);
end
rick=ricker(10,101,0.004,51*dt);
data_static=zeros(floor(tmax/dt+1),length(offset));
for j=1:size(impulse,2)
    data_static(:,j)=conv(impulse(:,j),rick,'same');
end
buff_rows=1024;
buff=zeros(buff_rows);
buff(1:size(data_static,1),1:size(data_static,2))=data_static(:,:);
C_static = fdct_usfft(buff,0);
img_static = fdct_usfft_dispcoef(C_static);

C_clean = C_data;
for i = 3:size(C_clean,2)-1
    for j = 1:size(C_clean{i},2)
        C_clean{i}{j}(:,:) = C_clean{i}{j}(:,:)-C_static{i}{j}(:,:);
    end
end
img_clean = fdct_usfft_dispcoef(C_clean);
data_clean = ifdct_usfft(C_clean,0);

%write gather
%writing the .H@ 
fileID=fopen('/scr1/ettore/events_clean.H@','w');
fwrite(fileID,data_clean(1:size(data,1),1:size(data,2)),'float32','s');
fclose(fileID);

%writing the .H 
fileID=fopen('events_clean.H','w');
fprintf(fileID,'%s\n','in=/scr1/ettore/events_clean.H@');
fprintf(fileID,'n1=%d\n',size(data,1));
fprintf(fileID,'o1=%d\n',0.);
fprintf(fileID,'d1=%d\n',dt);
fprintf(fileID,'n2=%d\n',size(data,2));
fprintf(fileID,'o2=%d\n',min(offset));
fprintf(fileID,'d2=%d\n',doff);
fprintf(fileID,'%s\n','label1="time [s]"');
fprintf(fileID,'%s\n','label2="offset [m]"');
fclose(fileID);



quit